/* IM2 Example */
/* zcc +zx -vn im2.c -lsplib2 -lmalloc -lndos */

#include <stdio.h>
#include <malloc.h>
#include <spritepack.h>
#pragma output STACKPTR=61440

/* Sprite Pack Variables Copied from "spritepack.h */

/* a rectangle that defines the screen area */
extern struct sp_Rect *sp_ClipStruct;
#asm
LIB SPCClipStruct
._sp_ClipStruct         defw SPCClipStruct
#endasm

/* Global Variables */

struct {
   uchar frames;
   uchar seconds;
   uchar minutes;
   uchar hours;
} time;

extern uchar *lastk;          /* system variable LAST_K */
#asm
   ._lastk  defw 23560;
#endasm

uchar stop = 0;               /* stop clock? */
struct sp_PSS ps;
uchar timestring[] = "\x16\x0b\x0b00:00:00:00\x00";
uchar brick[] = {0xff,0x22,0x22,0x22,0xff,0x88,0x88,0x88};
extern uchar arrowspr1[];
extern uchar arrowspr2[];

/* Keyboard Scanning Hook */

#asm
defw 0                        /* two free bytes before start of hook */
#endasm
int keyboard_isr(void)
{
   uchar c;

   if ((c = sp_GetKey()) != 0)
      *lastk = c;

   if (stop)
      return_c(0);
   return_nc(0);
}

/* Clock Hook */

#asm
defw 0                        /* two free bytes before start of hook */
#endasm
int clock_isr(void)
{
   if (++time.frames == 50) {
      time.frames = 0;
      if (++time.seconds == 60) {
         time.seconds = 0;
         if (++time.minutes == 60) {
            time.minutes = 0;
            if (++time.hours == 24)
               time.hours = 0;
         }
      }
   }
   return_nc(0);
}

/* user memory allocation policy uses C's malloc */

HEAPSIZE(256)            /* malloc's heap will be 256 bytes in size */
void *u_malloc = malloc;
void *u_free = free;

main()
{
   uchar i;
   uint key_S, key_s, x, y;
   struct sp_SS *ss;
   uchar *temp1, *temp2;

   /* Set Up IM2 Vectors -- comments follow listing */

   #asm
   di
   #endasm
   sp_InitIM2(0xf1f1);
   sp_CreateGenericISR(0xf1f1);
   sp_RegisterHook(255, keyboard_isr);
   sp_RegisterHookLast(255, clock_isr);
   sp_MouseAMXInit(0, 2);
   #asm
   ei
   #endasm

   heapinit(256);   /* initialize malloc heap */
   sp_TileArray(' ', brick);
   sp_Initialize(INK_RED | PAPER_WHITE, ' ');
   sp_Border(CYAN);

   ps.bounds = sp_ClipStruct;
   ps.flags = sp_PSS_INVALIDATE | sp_PSS_XWRAP | sp_PSS_YINC;
   ps.colour = INK_BLACK | PAPER_WHITE;

   /* Create a small colourless arrow pointer */

   ss = sp_CreateSpr(sp_MASK_SPRITE, 3, arrowspr1, 0, TRANSPARENT);
   sp_AddColSpr(ss, arrowspr2, TRANSPARENT);

   key_S = sp_LookupKey('S');
   key_s = sp_LookupKey('s');

   *lastk = 0;
   while (1) {

      sp_UpdateNow();

      /* Move Arrow Sprite */

      sp_MouseAMX(&x, &y, &i);
      sp_MoveSprAbs(ss, sp_ClipStruct, 0, y/8, x/8, x&7, y&7);

      /* Print the Time */

      temp1 = &time.hours; temp2 = timestring+3;
      while (*temp2) {
         *temp2 = (*temp1)/10+'0'; temp2++;
         *temp2 = (*temp1)%10+'0'; temp2++;
         temp1--; temp2++;
      }
      sp_PrintString(&ps, timestring);

      /* Show Results of Key Read */

      i = getk();
      if ((i > 31) && (i < 129))
         sp_PrintAtInv(1, 1, INK_BLUE | PAPER_YELLOW, i);

      /* Stop or Start the Clock */

      if (sp_KeyPressed(key_S))
         stop = 1;
      else if (sp_KeyPressed(key_s))
         stop = 0;
   }
}

/* Arrow Sprite */

#asm
defb 0,255,0,255,0,255,0,255,0,255,0,255,0,255   ; 7 blank pixels above sprite

._arrowspr1
defb @00000000, @00111111
defb @01000000, @00011111
defb @01100000, @00001111
defb @01110000, @00000111
defb @01111000, @00000011
defb @01111100, @00000001
defb @01111110, @00000000
defb @01101000, @00000000

defb @01001000, @00000011
defb @00000100, @00110001
defb @00000100, @11110001
defb @00000010, @11111000
defb @00000010, @11111000
defb @00000000, @11111100
defb @00000000, @11111111
defb @00000000, @11111111

defb 0,255,0,255,0,255,0,255,0,255,0,255,0,255,0,255   ; blank row below sprite
                                                ; also 7 pixels above arrowspr2
._arrowspr2
defb 0,255,0,255,0,255,0,255,0,255,0,255,0,255,0,255
defb 0,255,0,255,0,255,0,255,0,255,0,255,0,255,0,255

defb 0,255,0,255,0,255,0,255,0,255,0,255,0,255,0,255   ; blank row below sprite

#endasm
